'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var axios = require('axios');

class HackMDError extends Error {
    constructor(message) {
        super(message);
        Object.setPrototypeOf(this, new.target.prototype);
    }
}
class HttpResponseError extends HackMDError {
    code;
    statusText;
    constructor(message, code, statusText) {
        super(message);
        this.code = code;
        this.statusText = statusText;
    }
}
class MissingRequiredArgument extends HackMDError {
}
class InternalServerError extends HttpResponseError {
}
class TooManyRequestsError extends HttpResponseError {
    code;
    statusText;
    userLimit;
    userRemaining;
    resetAfter;
    constructor(message, code, statusText, userLimit, userRemaining, resetAfter) {
        super(message, code, statusText);
        this.code = code;
        this.statusText = statusText;
        this.userLimit = userLimit;
        this.userRemaining = userRemaining;
        this.resetAfter = resetAfter;
    }
}

const defaultOption = {
    unwrapData: true,
};
class API {
    accessToken;
    hackmdAPIEndpointURL;
    options;
    axios;
    constructor(accessToken, hackmdAPIEndpointURL = "https://api.hackmd.io/v1", options = {
        wrapResponseErrors: true,
        timeout: 30000,
        retryConfig: {
            maxRetries: 3,
            baseDelay: 100,
        },
    }) {
        this.accessToken = accessToken;
        this.hackmdAPIEndpointURL = hackmdAPIEndpointURL;
        this.options = options;
        if (!accessToken) {
            throw new MissingRequiredArgument('Missing access token when creating HackMD client');
        }
        this.axios = axios.create({
            baseURL: hackmdAPIEndpointURL,
            headers: {
                "Content-Type": "application/json",
            },
            timeout: options.timeout
        });
        this.axios.interceptors.request.use((config) => {
            config.headers.set('Authorization', `Bearer ${accessToken}`);
            return config;
        }, (err) => {
            return Promise.reject(err);
        });
        if (options.wrapResponseErrors) {
            this.axios.interceptors.response.use((response) => {
                return response;
            }, async (err) => {
                if (!err.response) {
                    return Promise.reject(err);
                }
                if (err.response.status >= 500) {
                    throw new InternalServerError(`HackMD internal error (${err.response.status} ${err.response.statusText})`, err.response.status, err.response.statusText);
                }
                else if (err.response.status === 429) {
                    throw new TooManyRequestsError(`Too many requests (${err.response.status} ${err.response.statusText})`, err.response.status, err.response.statusText, parseInt(err.response.headers['x-ratelimit-userlimit'], 10), parseInt(err.response.headers['x-ratelimit-userremaining'], 10), parseInt(err.response.headers['x-ratelimit-userreset'], 10));
                }
                else {
                    throw new HttpResponseError(`Received an error response (${err.response.status} ${err.response.statusText}) from HackMD`, err.response.status, err.response.statusText);
                }
            });
        }
        if (options.retryConfig) {
            this.createRetryInterceptor(this.axios, options.retryConfig.maxRetries, options.retryConfig.baseDelay);
        }
    }
    exponentialBackoff(retries, baseDelay) {
        return Math.pow(2, retries) * baseDelay;
    }
    isRetryableError(error) {
        return (!error.response ||
            (error.response.status >= 500 && error.response.status < 600) ||
            error.response.status === 429);
    }
    createRetryInterceptor(axiosInstance, maxRetries, baseDelay) {
        let retryCount = 0;
        axiosInstance.interceptors.response.use(response => response, async (error) => {
            if (retryCount < maxRetries && this.isRetryableError(error)) {
                const remainingCredits = parseInt(error.response?.headers['x-ratelimit-userremaining'], 10);
                if (isNaN(remainingCredits) || remainingCredits > 0) {
                    retryCount++;
                    const delay = this.exponentialBackoff(retryCount, baseDelay);
                    console.warn(`Retrying request... attempt #${retryCount} after delay of ${delay}ms`);
                    await new Promise(resolve => setTimeout(resolve, delay));
                    return axiosInstance(error.config);
                }
            }
            retryCount = 0; // Reset retry count after a successful request or when not retrying
            return Promise.reject(error);
        });
    }
    async getMe(options = defaultOption) {
        return this.unwrapData(this.axios.get("me"), options.unwrapData);
    }
    async getHistory(options = defaultOption) {
        return this.unwrapData(this.axios.get("history"), options.unwrapData);
    }
    async getNoteList(options = defaultOption) {
        return this.unwrapData(this.axios.get("notes"), options.unwrapData);
    }
    async getNote(noteId, options = defaultOption) {
        // Prepare request config with etag if provided in options
        const config = options.etag ? {
            headers: { 'If-None-Match': options.etag },
            // Consider 304 responses as successful
            validateStatus: (status) => (status >= 200 && status < 300) || status === 304
        } : undefined;
        const request = this.axios.get(`notes/${noteId}`, config);
        return this.unwrapData(request, options.unwrapData, true);
    }
    async createNote(payload, options = defaultOption) {
        return this.unwrapData(this.axios.post("notes", payload), options.unwrapData, true);
    }
    async updateNoteContent(noteId, content, options = defaultOption) {
        return this.unwrapData(this.axios.patch(`notes/${noteId}`, { content }), options.unwrapData, true);
    }
    async updateNote(noteId, payload, options = defaultOption) {
        return this.unwrapData(this.axios.patch(`notes/${noteId}`, payload), options.unwrapData, true);
    }
    async deleteNote(noteId, options = defaultOption) {
        return this.unwrapData(this.axios.delete(`notes/${noteId}`), options.unwrapData);
    }
    async getTeams(options = defaultOption) {
        return this.unwrapData(this.axios.get("teams"), options.unwrapData);
    }
    async getTeamNotes(teamPath, options = defaultOption) {
        return this.unwrapData(this.axios.get(`teams/${teamPath}/notes`), options.unwrapData);
    }
    async createTeamNote(teamPath, payload, options = defaultOption) {
        return this.unwrapData(this.axios.post(`teams/${teamPath}/notes`, payload), options.unwrapData);
    }
    async updateTeamNoteContent(teamPath, noteId, content) {
        return this.axios.patch(`teams/${teamPath}/notes/${noteId}`, { content });
    }
    async updateTeamNote(teamPath, noteId, options) {
        return this.axios.patch(`teams/${teamPath}/notes/${noteId}`, options);
    }
    async deleteTeamNote(teamPath, noteId) {
        return this.axios.delete(`teams/${teamPath}/notes/${noteId}`);
    }
    unwrapData(reqP, unwrap = true, includeEtag = false) {
        if (!unwrap) {
            // For raw responses, etag is available via response.headers
            return reqP;
        }
        return reqP.then(response => {
            const data = response.data;
            if (!includeEtag)
                return data;
            const etag = response.headers.etag || response.headers['ETag'];
            return { ...data, status: response.status, etag };
        });
    }
}

exports.API = API;
exports.default = API;
//# sourceMappingURL=index.cjs.map
